import java.util.*;

/**
 * Represents a left-bounded input/output tape for a Turing Machine
 */
public class TMTape {
	private List tape = new Vector();
	
	// Read symbol at position pos with 0 <= pos < tape.size
	public TMSymbol read(int pos) {
		TMSymbol result = null;
		try {
			result = (TMSymbol) tape.get(pos);
		} catch (IndexOutOfBoundsException ex) {
			System.out.println("*** Could not read from position " + 
							   pos + ", index out of bounds!");
			ex.printStackTrace();
		}
		return result;
	}
	
	// Write symbol at position pos, creating new space on right side of tape if necessary
	public void write(int pos, TMSymbol symbol) {
		try {
			if (0 <= pos && pos < tape.size())
				tape.set(pos, symbol);
			if (pos == tape.size())
				tape.add(pos, symbol);
		} catch (IndexOutOfBoundsException ex) {
			System.out.println("*** Could not write symbol " + symbol + 
							   " to position " + pos + ", index out of bounds!");
			ex.printStackTrace();
		}
	}
	
	// Clear the complete tape for new input/output
	public void clear() {
		tape.clear();
	}
	
	// Formatted output of the tape content
	public String toString() {
		String result = new String("|");
		ListIterator lit = tape.listIterator();
		while (lit.hasNext()) {
			result += lit.next() + "|";
		}
		return result;
	}
}
