/**
 * Test functionality of class TuringMachine
 *
 */
public class Test {

	public static void main(String[] args) {
		Test t = new Test();
		t.runTest();
	}
	
	// TM which processes input w of form "#LOL...#" and terminates with "L##...#" if
	// number of symbol L in w was odd, or "O##...#" otherwise (starting with rightmost symbol)
	private void runTest() {
		// Create TM with 4 states and 3 symbols
		TuringMachine tm = new TuringMachine(4, 3);
		// Get tape for later writing of input
		TMTape tape = tm.getTape();
		// Add 4 states with S0 as initial state
		tm.addState("S0", true);
		tm.addState("S1", false);
		tm.addState("S2", false);
		tm.addState("S3", false);
		// Add 3 symbols, one of them is the special bounding symbol declared as a constant in TMSymbol
		TMSymbol bound = tm.addSymbol(TMSymbol.BOUND);
		TMSymbol zero = tm.addSymbol("O");
		TMSymbol one = tm.addSymbol("L");
		// Add transitions in the order:
		// <originState>, <inputSymbol>, <targetState>, <outputSymbol>, <headDirection>
		tm.addTransition("S0", "O", "S0", "O", TuringMachine.RIGHT);
		tm.addTransition("S0", "L", "S0", "L", TuringMachine.RIGHT);
		tm.addTransition("S0", TMSymbol.BOUND, "S1", TMSymbol.BOUND, TuringMachine.LEFT);
		tm.addTransition("S1", "O", "S1", TMSymbol.BOUND, TuringMachine.LEFT);
		tm.addTransition("S1", "L", "S2", TMSymbol.BOUND, TuringMachine.LEFT);
		tm.addTransition("S1", TMSymbol.BOUND, "S3", "O", TuringMachine.HOLD);
		tm.addTransition("S2", "O", "S2", TMSymbol.BOUND, TuringMachine.LEFT);
		tm.addTransition("S2", "L", "S1", TMSymbol.BOUND, TuringMachine.LEFT);
		tm.addTransition("S2", TMSymbol.BOUND, "S3", "L", TuringMachine.HOLD);
		// First test with an odd number of L
		System.out.println("--- Odd number of symbol \"L\"");
		tape.write(0, bound);
		tape.write(1, one);
		tape.write(2, one);
		tape.write(3, zero);
		tape.write(4, one);
		tape.write(5, bound);
		// Initialize TM with position of rightmost non-bound symbol
		tm.init(4);
		// Execute TM with output of current state, current position and tape content in each step
		tm.execute(true);
		
		// Second test with an even number of L
		System.out.println("--- Even number of symbol \"L\"");
		tape.write(0, bound);
		tape.write(1, one);
		tape.write(2, one);
		tape.write(3, zero);
		tape.write(4, zero);
		tape.write(5, one);
		tape.write(6, one);
		tape.write(7, bound);
		tm.init(6);
		tm.execute(true);
	}
}
